# -*- coding: utf-8 -*-
"""KiCad Python Shell.

This module provides the python shell for KiCad.

Currently the shell is only available inside PCBNEW.

PCBNEW starts the shell once, by calling makePcbnewShellWindow() the
first time it is opened, subsequently the shell window is just hidden
or shown, as per user requirements.

IF makePcbnewShellWindow() is called again, a second/third shell window
can be created.  PCBNEW does not do this, but a user may call this from
the first shell if they require.

"""
import wx
import sys
import os

from wx.py import crust, editor, version, dispatcher
from wx.py.editor import EditorNotebook

import pcbnew

INTRO = "KiCAD:PCBNEW - Python Shell - PyAlaMode %s" % version.VERSION


class PcbnewPyShell(editor.EditorNotebookFrame):

    """The Pythonshell of PCBNEW."""

    def _setup_startup(self):
        """Initialise the startup script."""
        # Create filename for startup script.
        self.startup_file = os.path.join(self.config_dir,
                                         "PyShell_pcbnew_startup.py")
        self.execStartupScript = True

        # Check if startup script exists
        if not os.path.isfile(self.startup_file):
            # Not, so create a default.
            default_startup = open(self.startup_file, 'w')
            # provide the content for the default startup file.
            default_startup.write(
                "### DEFAULT STARTUP FILE FOR KiCad:PCBNEW Python Shell\n" +
                "# Enter any Python code you would like to execute when" +
                " the PCBNEW python shell first runs.\n" +
                "\n" +
                "# Eg:\n" +
                "\n" +
                "# import pcbnew\n" +
                "# board = pcbnew.GetBoard()\n")
            default_startup.close()

    def _setup(self):
        """
        Setup prior to first buffer creation.

        Called automatically by base class during init.
        """
        self.notebook = EditorNotebook(parent=self)
        intro = 'Py %s' % version.VERSION
        import imp
        module = imp.new_module('__main__')
        if sys.version_info >= (3,):
            import builtins
            module.__dict__['__builtins__'] = builtins
        else:
            import __builtin__
            module.__dict__['__builtins__'] = __builtin__
        namespace = module.__dict__.copy()

        self.config_dir = pcbnew.GetKicadConfigPath()
        self.dataDir = self.config_dir

        self._setup_startup()
        self.history_file = os.path.join(self.config_dir,
                                         "PyShell_pcbnew.history")

        self.config_file = os.path.join(self.config_dir,
                                        "PyShell_pcbnew.cfg")
        self.config = wx.FileConfig(localFilename=self.config_file)
        self.config.SetRecordDefaults(True)
        self.autoSaveSettings = False
        self.autoSaveHistory = False
        self.LoadSettings()

        # in case of wxPhoenix we need to create a wxApp first and store it
        # to prevent removal by gabage collector
        if 'phoenix' in wx.PlatformInfo:
            self.theApp = wx.App()

        self.crust = crust.Crust(parent=self.notebook,
                                 intro=intro, locals=namespace,
                                 rootLabel="locals()",
                                 startupScript=self.startup_file,
                                 execStartupScript=self.execStartupScript)

        self.shell = self.crust.shell
        # Override the filling so that status messages go to the status bar.
        self.crust.filling.tree.setStatusText = self.SetStatusText
        # Override the shell so that status messages go to the status bar.
        self.shell.setStatusText = self.SetStatusText
        # Fix a problem with the sash shrinking to nothing.
        self.crust.filling.SetSashPosition(200)
        self.notebook.AddPage(page=self.crust, text='*Shell*', select=True)
        self.setEditor(self.crust.editor)
        self.crust.editor.SetFocus()

        self.LoadHistory()

    def OnAbout(self, event):
        """Display an About window."""
        title = 'About : KiCad:PCBNEW - Python Shell'
        text = "Enhanced Python Shell for KiCad:PCBNEW\n\n" + \
               "This KiCad Python Shell is based on wxPython PyAlaMode.\n\n" + \
               "see: http://wiki.wxpython.org/PyAlaMode\n\n" + \
               "KiCad Revision: %s\n" % "??.??" + \
               "PyAlaMode Revision : %s\n" % version.VERSION + \
               "Platform: %s\n" % sys.platform + \
               "Python Version: %s\n" % sys.version.split()[0] + \
               "wxPython Version: %s\n" % wx.VERSION_STRING + \
               ("\t(%s)\n" % ", ".join(wx.PlatformInfo[1:]))

        dialog = wx.MessageDialog(self, text, title,
                                  wx.OK | wx.ICON_INFORMATION)
        dialog.ShowModal()
        dialog.Destroy()

    def EditStartupScript(self):
        """Open a Edit buffer of the startup script file."""
        self.bufferCreate(filename=self.startup_file)

    def LoadSettings(self):
        """Load settings for the shell."""
        if self.config is not None:
            editor.EditorNotebookFrame.LoadSettings(self, self.config)
            self.autoSaveSettings = \
                self.config.ReadBool('Options/AutoSaveSettings', False)
            self.execStartupScript = \
                self.config.ReadBool('Options/ExecStartupScript', True)
            self.autoSaveHistory = \
                self.config.ReadBool('Options/AutoSaveHistory', False)
            self.hideFoldingMargin = \
                self.config.ReadBool('Options/HideFoldingMargin', True)

    def SaveSettings(self, force=False):
        """
        Save settings for the shell.

        Arguments:

        force -- False - Autosaving.  True - Manual Saving.
        """
        if self.config is not None:
            # always save these
            self.config.WriteBool('Options/AutoSaveSettings',
                                  self.autoSaveSettings)
            if self.autoSaveSettings or force:
                editor.EditorNotebookFrame.SaveSettings(self, self.config)

                self.config.WriteBool('Options/AutoSaveHistory',
                                      self.autoSaveHistory)
                self.config.WriteBool('Options/ExecStartupScript',
                                      self.execStartupScript)
                self.config.WriteBool('Options/HideFoldingMargin',
                                      self.hideFoldingMargin)
            if self.autoSaveHistory:
                self.SaveHistory()

    def DoSaveSettings(self):
        """Menu function to trigger saving the shells settings."""
        if self.config is not None:
            self.SaveSettings(force=True)
            self.config.Flush()

    def SaveHistory(self):
        """Save shell history to the shell history file."""
        if self.dataDir:
            try:
                name = self.history_file
                f = file(name, 'w')
                hist = []
                enc = wx.GetDefaultPyEncoding()
                for h in self.shell.history:
                    if isinstance(h, unicode):
                        h = h.encode(enc)
                    hist.append(h)
                hist = '\x00\n'.join(hist)
                f.write(hist)
                f.close()
            except:
                d = wx.MessageDialog(self, "Error saving history file.",
                                     "Error", wx.ICON_EXCLAMATION | wx.OK)
                d.ShowModal()
                d.Destroy()
                raise

    def LoadHistory(self):
        """Load shell history from the shell history file."""
        if self.dataDir:
            name = self.history_file
            if os.path.exists(name):
                try:
                    f = file(name, 'U')
                    hist = f.read()
                    f.close()
                    self.shell.history = hist.split('\x00\n')
                    dispatcher.send(signal="Shell.loadHistory",
                                    history=self.shell.history)
                except:
                    d = wx.MessageDialog(self,
                                         "Error loading history file!",
                                         "Error", wx.ICON_EXCLAMATION | wx.OK)
                    d.ShowModal()
                    d.Destroy()


def makePcbnewShellWindow(parent=None):
    """
    Create a new Shell Window and return its handle.

    Arguments:
    parent -- The parent window to attach to.

    Returns:
    The handle to the new window.
    """
    pyshell = PcbnewPyShell(parent, id=-1, title=INTRO)
    pyshell.Show()
    return pyshell



#!/usr/bin/env python2


# Author: Dick Hollenbeck
# Report any length problems pertaining to a SDRAM DDR3 T topology using
# 4 memory chips: a T into 2 Ts routing strategy from the CPU.

# Designed to be run from the command line in a process separate from pcbnew.
# It can monitor writes to disk which will trigger updates to its output, or
# it can be run with --once option.


from __future__ import print_function

import pcbnew
import os.path
import sys
import time


CPU_REF = 'U7'      # CPU reference designator

# four SDRAM memory chips:
DDR_LF  = 'U15'     # left front DRAM
DDR_RF  = 'U17'     # right front DRAM
DDR_LB  = 'U16'     # left back DRAM
DDR_RB  = 'U18'     # right back DRAM


# Length of SDRAM clock, it sets the maximum or equal needed for other traces
CLOCK_LEN = pcbnew.FromMils( 2.25 * 1000 )

def addr_line_netname(line_no):
    """From an address line number, return the netname"""
    netname = '/DDR3/DRAM_A' + str(line_no)
    return netname

# Establish GOALS which are LENs, TOLERANCEs and NETs for each group of nets.

# Net Group: ADDR_AND_CMD
ADDR_AND_CMD_LEN = pcbnew.FromMils( 2.22 * 1000 )
ADDR_AND_CMD_TOLERANCE = pcbnew.FromMils( 25 ) / 2
ADDR_AND_CMD_NETS = [addr_line_netname(a) for a in range(0,16)]
ADDR_AND_CMD_NETS += [
    '/DDR3/DRAM_SDBA0',
    '/DDR3/DRAM_SDBA1',
    '/DDR3/DRAM_SDBA2',
    '/DDR3/DRAM_RAS_B',
    '/DDR3/DRAM_CAS_B',
    '/DDR3/DRAM_WE_B'
    ]


# Net Group: CONTROL
CONTROL_LEN = pcbnew.FromMils( 2.10 * 1000 )
CONTROL_TOLERANCE = pcbnew.FromMils( 50 ) / 2
CONTROL_NETS = [
    '/DDR3/DRAM_SDODT0',
    #'/DDR3/DRAM_SDODT1',
    '/DDR3/DRAM_CS0_B',
    #'/DDR3/DRAM_CS1_B',
    '/DDR3/DRAM_SDCKE0',
    #'/DDR3/DRAM_SDCKE1',
    ]



BRIGHTGREEN = '\033[92;1m'
GREEN = '\033[92m'
BRIGHTRED = '\033[91;1m'
RED = '\033[91m'
ENDC = '\033[0m'


pcb = None
nets = None

dbg_conn = False        # when true prints out reason for track discontinuity

def print_color(color, s):
    print(color + s + ENDC)


def addr_line_netname(line_no):
    netname = '/DDR3/DRAM_A' + str(line_no)
    return netname


def pad_name(pad):
    return str( pad.GetParent().Reference().GetShownText() ) + '/' + pad.GetPadName()


def pad_pos(pad):
    return str(pad.GetPosition())


def pads_in_net(netname):
    byname = {}
    pads = nets[netname].Pads()
    for pad in pads:
        byname[pad_name(pad)] = pad
    return byname


def track_ends(track):
    """return a string showing both ends of a track"""
    return str(track.GetStart()) + ' ' + str(track.GetEnd())


def print_tracks(net_name,tracks):
    print('net:', net_name)
    for track in tracks:
        print(' track:', track_ends(track))


def sum_track_lengths(point1,point2,netcode):
    tracks = pcb.TracksInNetBetweenPoints(point1, point2, netcode)
    sum = 0
    for t in tracks:
        sum += t.GetLength()
    return sum


def tracks_in_net(netname):
    nc = pcb.GetNetcodeFromNetname(netname)
    tracks_and_vias = pcb.TracksInNet(nc)
    # remove vias while making new non-owning list
    tracks = [t for t in tracks_and_vias if not t.Type() == pcbnew.PCB_VIA_T]
    return tracks


def print_pad(pad):
    print( " pad name:'%s' pos:%s" % ( pad_name(pad), pad_pos(pad) ) )


def print_pads(prompt,pads):
    print(prompt)
    for pad in pads:
        print_pad(pad)


def is_connected(start_pad, end_pad):
    """
    Return True if the two pads are copper connected only with vias and tracks
    directly and with no intervening pads, else False.
    """
    netcode = start_pad.GetNet().GetNet()
    try:
        tracks = pcb.TracksInNetBetweenPoints(start_pad.GetPosition(), end_pad.GetPosition(), netcode)
    except IOError as ioe:
        if dbg_conn:        # can be True when wanting details on discontinuity
            print(ioe)
        return False
    return True


def find_connected_pad(start_pad, pads):
    for p in pads:
        if p == start_pad:
            continue
        if is_connected(start_pad,p):
            return p
    raise IOError( 'no connection to pad %s' % pad_name(start_pad) )


def find_cpu_pad(pads):
    for p in pads:
        if CPU_REF in pad_name(p):
            return p
    raise IOError( 'no cpu pad' )


def report_teed_lengths(groupname, netname, target_length, tolerance):
    global dbg_conn

    print(groupname, netname)
    nc = pcb.GetNetcodeFromNetname(netname)
    #print("nc", nc)

    pads = nets[netname].Pads()

    # convert from std::vector<> to python list
    pads = list(pads)
    #print_pads(netname, pads )

    cpu_pad = find_cpu_pad(pads)
    pads.remove(cpu_pad)

    # a trap for a troublesome net that appears to be disconnected or has stray segments.
    if netname == None:
    #if netname == '/DDR3/DRAM_SDCKE0':
        dbg_conn = True

    # find the first T
    #print_pads(netname + ' without cpu pad', pads )
    t1 = find_connected_pad(cpu_pad, pads)
    pads.remove(t1)

    # find 2 second tier T pads
    t2_1 = find_connected_pad(t1, pads)
    pads.remove(t2_1)

    t2_2 = find_connected_pad(t1, pads)
    pads.remove(t2_2)

    cpad = [0] * 4

    # find 4 memory pads off of each 2nd tier T
    cpad[0] = find_connected_pad(t2_1, pads)
    pads.remove(cpad[0])

    cpad[1] = find_connected_pad(t2_1, pads)
    pads.remove(cpad[1])

    cpad[2] = find_connected_pad(t2_2, pads)
    pads.remove(cpad[2])

    cpad[3] = find_connected_pad(t2_2, pads)
    pads.remove(cpad[3])

    len_t1   = sum_track_lengths(cpu_pad.GetPosition(),t1.GetPosition(),nc)
    #print("len_t1 %.0f" % len_t1)

    len_t2_1 = sum_track_lengths(t1.GetPosition(),t2_1.GetPosition(),nc)
    len_t2_2 = sum_track_lengths(t1.GetPosition(),t2_2.GetPosition(),nc)
    #print("len_t2_1 %.0f" % len_t2_1)
    #print("len_t2_2 %.0f" % len_t2_2)

    lens = [0] * 4

    lens[0] = sum_track_lengths(t2_1.GetPosition(),cpad[0].GetPosition(),nc)
    lens[1] = sum_track_lengths(t2_1.GetPosition(),cpad[1].GetPosition(),nc)
    lens[2] = sum_track_lengths(t2_2.GetPosition(),cpad[2].GetPosition(),nc)
    lens[3] = sum_track_lengths(t2_2.GetPosition(),cpad[3].GetPosition(),nc)

    """
    for index, total_len in enumerate(lens):
        print( "%s: %.0f" % (pad_name(cpad[index]), lens[index]))
    """

    # Each net goes from CPU to four memory chips, these are the 4 lengths from
    # CPU to each of the for memory chip balls/pads, some of these journies are
    # common with one another but branch off at each T.
    lens[0] += len_t1 + len_t2_1
    lens[1] += len_t1 + len_t2_1
    lens[2] += len_t1 + len_t2_2
    lens[3] += len_t1 + len_t2_2

    for index, total_len in enumerate(lens):
        delta = total_len - target_length
        if delta > tolerance:
            print_color( BRIGHTRED, "%s %s len:%.0f long by %.0f mils" %
                  (netname, pad_name(cpad[index]), pcbnew.ToMils(total_len), pcbnew.ToMils(delta - tolerance)  ))
        elif delta < -tolerance:
            print_color( BRIGHTRED, "%s %s len:%.0f short by %.0f mils" %
                  (netname, pad_name(cpad[index]), pcbnew.ToMils(total_len), pcbnew.ToMils(tolerance - delta) ))



def load_board_and_report_lengths(filename):

    global pcb
    pcb = pcbnew.LoadBoard(filename)
    pcb.BuildListOfNets()               # required so 'pcb' contains valid netclass data

    global nets
    nets = pcb.GetNetsByName()

    for netname in ADDR_AND_CMD_NETS:
        report_teed_lengths("addr_and_cmd", netname, ADDR_AND_CMD_LEN, ADDR_AND_CMD_TOLERANCE)

    for netname in CONTROL_NETS:
        report_teed_lengths("control", netname, CONTROL_LEN, CONTROL_TOLERANCE)



if __name__ == "__main__":
    try:
        boardfile = sys.argv[1]
    except IndexError:
        print("Usage: %s <boardname.kicad_pcb> [--once]" % sys.argv[0])
        sys.exit(1)

    first = True

    while True:

        # wait for the file contents to change
        lastmtime = os.path.getmtime(boardfile)
        mtime = lastmtime
        while mtime == lastmtime and not first:
            try:
                mtime = os.path.getmtime(boardfile)
            except OSError:
                pass # kicad save process seems to momentarily delete file, so there's a race here with "No such file.."
            time.sleep(0.5)

        # The "Debug" build of pcbnew writes to disk slowy, new file takes time to get to disk.
        time.sleep(1)

        first = False

        print( '\033[2J' )  # clear screen, maybe

        load_board_and_report_lengths(boardfile)

        if "--once" in sys.argv:
            sys.exit(0)



#
# Example python script to generate a BOM from a KiCad generic netlist
#
# Example: Sorted and Grouped CSV BOM
#
"""
    @package
    Generate a csv BOM list.
    Components are sorted by ref and grouped by value
    Fields are (if exist)
    Item, Qty, Reference(s), Value, LibPart, Footprint, Datasheet

    Command line:
    python "pathToFile/bom_csv_grouped_by_value.py" "%I" "%O.csv"
"""

from __future__ import print_function

# Import the KiCad python helper module and the csv formatter
import kicad_netlist_reader
import csv
import sys

def myEqu(self, other):
    """myEqu is a more advanced equivalence function for components which is
    used by component grouping. Normal operation is to group components based
    on their value and footprint.

    In this example of a custom equivalency operator we compare the
    value, the part name and the footprint.
    """
    result = True
    if self.getValue() != other.getValue():
        result = False
    elif self.getPartName() != other.getPartName():
        result = False
    elif self.getFootprint() != other.getFootprint():
        result = False

    return result

# Override the component equivalence operator - it is important to do this
# before loading the netlist, otherwise all components will have the original
# equivalency operator.
kicad_netlist_reader.comp.__eq__ = myEqu

if len(sys.argv) != 3:
    print("Usage ", __file__, "<generic_netlist.xml> <output.csv>", file=sys.stderr)
    sys.exit(1)


# Generate an instance of a generic netlist, and load the netlist tree from
# the command line option. If the file doesn't exist, execution will stop
net = kicad_netlist_reader.netlist(sys.argv[1])

# Open a file to write to, if the file cannot be opened output to stdout
# instead
try:
    f = open(sys.argv[2], 'w')
except IOError:
    e = "Can't open output file for writing: " + sys.argv[2]
    print( __file__, ":", e, sys.stderr )
    f = sys.stdout

# subset the components to those wanted in the BOM, controlled
# by <configure> block in kicad_netlist_reader.py
components = net.getInterestingComponents()

compfields = net.gatherComponentFieldUnion(components)
partfields = net.gatherLibPartFieldUnion()

# remove Reference, Value, Datasheet, and Footprint, they will come from 'columns' below
partfields -= set( ['Reference', 'Value', 'Datasheet', 'Footprint'] )

columnset = compfields | partfields     # union

# prepend an initial 'hard coded' list and put the enchillada into list 'columns'
columns = ['Item', 'Qty', 'Reference(s)', 'Value', 'LibPart', 'Footprint', 'Datasheet'] + sorted(list(columnset))

# Create a new csv writer object to use as the output formatter
out = csv.writer( f, lineterminator='\n', delimiter=',', quotechar='\"', quoting=csv.QUOTE_ALL )

# override csv.writer's writerow() to support encoding conversion (initial encoding is utf8):
def writerow( acsvwriter, columns ):
    utf8row = []
    for col in columns:
        utf8row.append( str(col) )  # currently, no change
    acsvwriter.writerow( utf8row )

# Output a set of rows as a header providing general information
writerow( out, ['Source:', net.getSource()] )
writerow( out, ['Date:', net.getDate()] )
writerow( out, ['Tool:', net.getTool()] )
writerow( out, ['Generator:', sys.argv[0]] )
writerow( out, ['Component Count:', len(components)] )
writerow( out, [] )
writerow( out, ['Individual Components:'] )
writerow( out, [] )                        # blank line
writerow( out, columns )

# Output all the interesting components individually first:
row = []
for c in components:
    del row[:]
    row.append('')                                      # item is blank in individual table
    row.append('')                                      # Qty is always 1, why print it
    row.append( c.getRef() )                            # Reference
    row.append( c.getValue() )                          # Value
    row.append( c.getLibName() + ":" + c.getPartName() ) # LibPart
    #row.append( c.getDescription() )
    row.append( c.getFootprint() )
    row.append( c.getDatasheet() )

    # from column 7 upwards, use the fieldnames to grab the data
    for field in columns[7:]:
        row.append( c.getField( field ) );

    writerow( out, row )


writerow( out, [] )                        # blank line
writerow( out, [] )                        # blank line
writerow( out, [] )                        # blank line

writerow( out, ['Collated Components:'] )
writerow( out, [] )                        # blank line
writerow( out, columns )                   # reuse same columns



# Get all of the components in groups of matching parts + values
# (see kicad_netlist_reader.py)
grouped = net.groupComponents(components)


# Output component information organized by group, aka as collated:
item = 0
for group in grouped:
    del row[:]
    refs = ""

    # Add the reference of every component in the group and keep a reference
    # to the component so that the other data can be filled in once per group
    for component in group:
        if len(refs) > 0:
            refs += ", "
        refs += component.getRef()
        c = component

    # Fill in the component groups common data
    # columns = ['Item', 'Qty', 'Reference(s)', 'Value', 'LibPart', 'Footprint', 'Datasheet'] + sorted(list(columnset))
    item += 1
    row.append( item )
    row.append( len(group) )
    row.append( refs );
    row.append( c.getValue() )
    row.append( c.getLibName() + ":" + c.getPartName() )
    row.append( net.getGroupFootprint(group) )
    row.append( net.getGroupDatasheet(group) )

    # from column 7 upwards, use the fieldnames to grab the data
    for field in columns[7:]:
        row.append( net.getGroupField(group, field) );

    writerow( out, row  )

f.close()



#
# Example python script to generate a BOM from a KiCad generic netlist
#
# Example: Sorted and Grouped CSV BOM
#

"""
    @package
    Generate a Tab delimited list (csv file type).
    Components are sorted by ref and grouped by value with same footprint
    Fields are (if exist)
    'Ref', 'Qnty', 'Value', 'Cmp name', 'Footprint', 'Description', 'Vendor'

    Command line:
    python "pathToFile/bom_csv_grouped_by_value_with_fp.py" "%I" "%O.csv"
"""

# Import the KiCad python helper module and the csv formatter
import kicad_netlist_reader
import csv
import sys

# Generate an instance of a generic netlist, and load the netlist tree from
# the command line option. If the file doesn't exist, execution will stop
net = kicad_netlist_reader.netlist(sys.argv[1])

# Open a file to write to, if the file cannot be opened output to stdout
# instead
try:
    f = open(sys.argv[2], 'w')
except IOError:
    e = "Can't open output file for writing: " + sys.argv[2]
    print(__file__, ":", e, sys.stderr)
    f = sys.stdout

# Create a new csv writer object to use as the output formatter
out = csv.writer(f, lineterminator='\n', delimiter=',', quotechar='\"', quoting=csv.QUOTE_ALL)

# Output a set of rows for a header providing general information
out.writerow(['Source:', net.getSource()])
out.writerow(['Date:', net.getDate()])
out.writerow(['Tool:', net.getTool()])
out.writerow( ['Generator:', sys.argv[0]] )
out.writerow(['Component Count:', len(net.components)])
out.writerow(['Ref', 'Qnty', 'Value', 'Cmp name', 'Footprint', 'Description', 'Vendor'])

# Get all of the components in groups of matching parts + values
# (see ky_generic_netlist_reader.py)
grouped = net.groupComponents()

# Output all of the component information
for group in grouped:
    refs = ""

    # Add the reference of every component in the group and keep a reference
    # to the component so that the other data can be filled in once per group
    for component in group:
        refs += component.getRef() + ", "
        c = component

    # Fill in the component groups common data
    out.writerow([refs, len(group), c.getValue(), c.getPartName(), c.getFootprint(),
        c.getDescription(), c.getField("Vendor")])





#
# Example python script to generate a BOM from a KiCad generic netlist
#
"""
    @package
    Generate a BOM list file (a simple text).
    Components are sorted by ref
    One component per line
    Fields are (if exist)
    Ref, Quantity, value, Part, footprint, Description, Vendor
    Fields are separated by tabs

    Command line:
    python "pathToFile/bom_sorted_by_ref.py" "%I" "%O.txt"
"""

from __future__ import print_function

# Import the KiCad python helper module and the csv formatter
import kicad_netlist_reader
import csv
import sys

# Generate an instance of a generic netlist, and load the netlist tree from
# the command line option. If the file doesn't exist, execution will stop
net = kicad_netlist_reader.netlist(sys.argv[1])

# Open a file to write to, if the file cannot be opened output to stdout
# instead
try:
    f = open(sys.argv[2], 'w')
except IOError:
    e = "Can't open output file for writing: " + sys.argv[2]
    print(__file__, ":", e, sys.stderr)
    f = sys.stdout

# Create a new csv writer object to use as the output formatter, although we
# are created a tab delimited list instead!
out = csv.writer(f, lineterminator='\n', delimiter='\t', quotechar="\"", quoting=csv.QUOTE_ALL)

# override csv.writer's writerow() to support utf8 encoding:
def writerow( acsvwriter, columns ):
    utf8row = []
    for col in columns:
        txt=str(col);
        utf8row.append( txt )
    acsvwriter.writerow( utf8row )

components = net.getInterestingComponents()

# Output a field delimited header line
writerow( out, ['Source:', net.getSource()] )
writerow( out, ['Date:', net.getDate()] )
writerow( out, ['Tool:', net.getTool()] )
writerow( out, ['Component Count:', len(components)] )
writerow( out, ['Ref', 'Value', 'Part', 'Footprint', 'Description', 'Vendor'] )

# Output all of the component information
for c in components:
    writerow( out, [c.getRef(), c.getValue(), c.getLibName() + ":" + c.getPartName(),
        c.getFootprint(), c.getDescription(), c.getField("Vendor")])




from pcbnew import *
import random
import pprint


class testundoredo0(ActionPlugin):

    def defaults(self):
        self.name = "Test Undo/Redo: Remove footprints"
        self.category = "Test Undo/Redo"
        self.description = ""

    def Run(self):
        pcb = GetBoard()

        for module in pcb.GetModules():
            pcb.RemoveNative(module)


class testundoredo1(ActionPlugin):

    def defaults(self):
        self.name = "Test Undo/Redo: Remove all board items"
        self.category = "Test Undo/Redo"
        self.description = ""

    def Run(self):
        pcb = GetBoard()

        while pcb.GetAreaCount() > 0:
            area = pcb.GetArea(0)
            pcb.RemoveNative(area)

        for module in pcb.GetModules():
            pcb.RemoveNative(module)

        for track in pcb.GetTracks():
            pcb.RemoveNative(track)

        for draw in pcb.m_Drawings:
            pcb.RemoveNative(draw)


class testundoredo2(ActionPlugin):

    def defaults(self):
        self.name = "Test Undo/Redo: Generate random content"
        self.category = "Test Undo/Redo"
        self.description = ""

    def createFPCXModule(self,pads):
        size_025_160mm = wxSizeMM(0.25,1.6)
        size_150_200mm = wxSizeMM(1.50,2.0)
        # create a new module, it's parent is our previously created pcb
        module = MODULE(self.pcb)
        module.SetReference("FPC"+str(pads))   # give it a reference name
        module.Reference().SetPosition(wxPointMM(-1,-1))
        self.pcb.Add(module)             # add it to our pcb
        m_pos = wxPointMM(0,0)#random.randint(10,200),random.randint(10,200))
        module.SetPosition(m_pos)

        # create a pad array and add it to the module


        def smdRectPad(module,size,pos,name):
            pad = D_PAD(module)
            pad.SetSize(size)
            pad.SetShape(PAD_SHAPE_RECT)
            pad.SetAttribute(PAD_ATTRIB_SMD)
            pad.SetLayerSet(pad.SMDMask())
            pad.SetPosition(pos)
            pad.SetPadName(name)
            return pad

        for n in range (0,pads):
            pad = smdRectPad(module,size_025_160mm,wxPointMM(0.5*n,0),str(n+1))
            module.Add(pad)


        pad_s0 = smdRectPad(module,size_150_200mm,wxPointMM(-1.6,1.3),"0")
        pad_s1 = smdRectPad(module,size_150_200mm,wxPointMM((pads-1)*0.5+1.6,1.3),"0")
        module.Add(pad_s0)
        module.Add(pad_s1)

        e = EDGE_MODULE(module)
        e.SetStart0(wxPointMM(-1,0))
        e.SetEnd0(wxPointMM(0,0))
        e.SetWidth(FromMM(0.2))
        e.SetLayer(F_SilkS)
        e.SetShape(S_SEGMENT)
        module.Add(e)
        module.SetPosition(wxPointMM(random.randint(20,200),random.randint(20,150)))
        return module



    def Run(self):
        self.pcb = GetBoard()
        random.seed()

        for i in range(10):
            seg = DRAWSEGMENT()
            seg.SetLayer( random.choice([Edge_Cuts,Cmts_User,Eco1_User,Eco2_User]) )
            seg.SetStart( wxPointMM( random.randint(10,100),
                                random.randint(10,100) ) )
            seg.SetEnd( wxPointMM( random.randint(10,100),
                                random.randint(10,100) ) )
            self.pcb.Add( seg )

            if i%2 == 0:
                t = TRACK(None)
            else:
                t = VIA(None)
                #t.SetLayerPair(segments['layerPair'][0],segments['layerPair'][1])
                t.SetViaType(VIA_THROUGH)
                t.SetDrill(FromMM(random.randint(1,20)/10.0))
            t.SetStart(wxPointMM(random.randint(100,150),random.randint(100,150)))
            t.SetEnd(wxPointMM(random.randint(100,150),random.randint(100,150)))
            t.SetWidth(FromMM(random.randint(1,15)/10.0))
            t.SetLayer(random.choice([F_Cu,B_Cu]))
            self.pcb.Add(t)

            self.createFPCXModule(random.randint(2,40))


class testundoredo3(ActionPlugin):

    def defaults(self):
        self.name = "Test Undo/Redo: Move elements randomly"
        self.category = "Test Undo/Redo"
        self.description = ""

    def Run(self):
        pcb = GetBoard()

        for i in range(0,pcb.GetAreaCount()):
            area = pcb.GetArea(i)
            area.Move(wxPointMM(random.randint(-20,20),random.randint(-20,20)))

        for module in pcb.GetModules():
            module.Move(wxPointMM(random.randint(-20,20),random.randint(-20,20)))
            if random.randint(0,10) > 5:
                module.Flip(module.GetPosition())

        for track in pcb.GetTracks():
            track.Move(wxPointMM(random.randint(-20,20),random.randint(-20,20)))

        for draw in pcb.m_Drawings:
            draw.Move(wxPointMM(random.randint(-20,20),random.randint(-20,20)))



testundoredo0().register()
testundoredo1().register()
testundoredo2().register()
testundoredo3().register()
